/*
 * Reveal | historythumb.js
 * Copyright 2005 by Michael Wu
 * Released under GPL v2
 */

var canvas_counter = 0;

function historyEntry (source)
{
this.thumb = null;
this.src = source;
this.title = "";
this.uri = "about:blank";
this.pagetext = "";
this.host = "about:blank";
this.box = null;
}

// Maintains data for a single session history (tab)
function historyThumbCache (b)
{
	this.browser = b;
	this.thumbnails = new Array(); // actually stores historyEntrys
	this.index = -1;
	this.canvas_cache = new Array();
	this.cache_fill = 0;
	this.text = null;
	b.thumbcache = this;

	// stupid hack - needed when user has homepage set to about:blank
	// for some reason, sessionHistory is not initialized if that is the case
	if (b.sessionHistory != null) 
		b.sessionHistory.addSHistoryListener(this);
	else
		setTimeout(function(self) {
			self.browser.sessionHistory.addSHistoryListener(self);
			if (self.browser.sessionHistory.index >= 0)
				self.changeIndex(self.browser.sessionHistory.index);
		}, 10, this);

	this.firstRun = 1;
	this.changeIndex(0);
	b.addEventListener("pageshow", this.updateListener, false);
};

historyThumbCache.prototype = {

QueryInterface: function(aIID) {
	if (aIID.equals(Components.interfaces.nsISHistoryListener) ||
	    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
	    aIID.equals(Components.interfaces.nsISupports))
		return this;

	throw Components.results.NS_NOINTERFACE;
},

concatenateText: function (nl) {
	var i, item, name;
	for (i = 0; i < nl.length; i++) {
		item = nl.item(i);
		// avoid adding script/noscript source
		name = item.nodeName.toLowerCase();
		if (name == "script" || name == "noscript")
			continue;

		if (item.nodeType == Node.TEXT_NODE) {
			this.text += item.nodeValue;
			continue;
		}

		if (name == "frameset")
			this.concatenateFrames(item.childNodes);
		else if (name == "iframe" &&
			 item.contentDocument != null &&
			 item.contentDocument.body != null)
			this.concatenateWhatever(item.contentDocument.body);
		else
			this.concatenateText(item.childNodes);
	}
},

concatenateFrames: function (nl) {
	var i, item, name;
	for (i = 0; i < nl.length; i++) {
		item = nl.item(i);
		name = item.nodeName.toLowerCase();
		switch (name) {
		case "frameset":
			this.concatenateFrames(item.childNodes);
			break;
		case "frame":
			if (item.contentDocument.body != null)
				this.concatenateText(item.contentDocument.body.childNodes);
			break;
		case "body":
			this.concatenateText(item.childNodes);
			break;
		}
	}
},

concatenateWhatever: function (node) {
	if (node.nodeName.toLowerCase() == "frameset")
		this.concatenateFrames(node.childNodes);
	else
		this.concatenateText(node.childNodes);
},

updatePagetext: function () {
	if (!HistoryThumb.cache_pagetext)
		return;

	if (this.index < 0)
		return;

	var hentry = this.thumbnails[this.index];
	if (this.browser.contentDocument == null ||
	    this.browser.contentDocument.body == null) {
		hentry.pagetext = "";
	} else {
		var body = this.browser.contentDocument.body;
		this.text = "";
		this.concatenateWhatever(body);
		hentry.pagetext = this.text;
	}
},

delayedRender: function (self) {
	delete self.timerID;

	self.makeThumbnail();
	self.notifyListeners(false);
},

scheduleRender: function (delay) {
	if (this.timerID) {
		clearTimeout(this.timerID);
		delete this.timerID;
	}

	this.timerID = setTimeout(this.delayedRender, delay, this);
},

updateEntryURI: function (hentry, uri) {
	hentry.uri = uri.spec;
	try {
		hentry.host = uri.host;
	} catch (e) {
		// there is a glorious failure when the URI doesn't have
		// a host component, like in the case of about:blank.
		// we have to catch the exception..
		hentry.host = uri.spec;
	}
},

makeThumbnail: function () {
	if (this.timerID) {
		clearTimeout(this.timerID);
		delete this.timerID;
	}

	if (this.index < 0)
		return;

	var hentry = this.thumbnails[this.index];
	if (this.browser.currentURI != null &&
	    this.browser.currentURI.spec == "about:blank") {
		hentry.thumb.style.display = "none";
		return;
	}

	hentry.thumb.style.display = "block";
	hentry.title = this.browser.contentTitle;
	this.updateEntryURI(hentry, this.browser.currentURI);

	var thumb = hentry.thumb;
	var width, height;
	var maxlen = HistoryThumb.small_maxwidth;

	if (this.browser.contentDocument instanceof ImageDocument) {
		var img = this.browser.contentDocument.imageRequest.decoderObserver;
		if (img.width <= maxlen && img.height <= maxlen) {
			x = img.width;
			y = img.height;
		} else if (img.width > img.height) {
			x = maxlen;
			y = (maxlen*img.height)/img.width;
		} else {
			y = maxlen;
			x = (maxlen*img.width)/img.height;
		}
		
		thumb.width = Math.ceil(x);
		thumb.height = Math.ceil(y);

		var ctx = thumb.getContext("2d");
		ctx.fillStyle = "rgb(255, 255, 255)";
		ctx.fillRect(0, 0, thumb.width, thumb.height);
		// note: at the moment, drawImage has some serious issues
		// with certain images that have oddly shaped transparent parts
		ctx.drawImage(img, 0, 0, thumb.width, thumb.height);
		return;
	}

	var x, y;
	var sleft, stop;

	// note that there is no clientWidth/Height if the document is XUL
	width = this.browser.contentDocument.documentElement.clientWidth;
	height = this.browser.contentDocument.documentElement.clientHeight;

	// a tiny percentage of sites out there give bogus clientHeight in documentElement
	if (this.browser.contentDocument.body != null) {
		//if (width == null ||
		//    width < this.browser.contentDocument.body.clientWidth)
		//	width = this.browser.contentDocument.body.clientWidth;

		if (height == null ||
		    this.browser.contentDocument.body.offsetHeight !=
		    this.browser.contentDocument.body.scrollHeight)
			height = this.browser.contentDocument.body.clientHeight;
	}

	// why can boxObject be missing? stop messing with me, damnit.
	// I also hate how clientWidth/Height can be zero. wtf?
	if (this.browser.boxObject != null) {
		// This section does two things:
		// 1. clip the thumbnail to the size of the viewport if
		// the page is bigger than the viewport
		// 2. set the thumbnail to the size of the viewport if
		// clientWidth/Height is 0 or this isn't a webpage (null)
		if (width > this.browser.boxObject.width ||
		    width <= 0 || width == null)
			width = this.browser.boxObject.width;

		if (height > this.browser.boxObject.height ||
		    height <= 0 || height == null)
			height = this.browser.boxObject.height;
	} else if (width <= 0 || height <= 0 ||
		   width == null || height == null) {
		if (HT_warning)
			alert("This page <" + this.browser.currentURI.spec +"> cannot be thumbnailed. \n\n Please report this site to the extension author at <flamingice@sourmilk.net>. Thanks!");

		HT_warning = false;
	}

	// fit everything inside a maxlen x maxlen sized rectangle
	if (width <= maxlen && height <= maxlen) {
		x = width;
		y = height;
	} else if (width > height) {
		x = maxlen;
		y = (maxlen*height)/width;
	} else {
		y = maxlen;
		x = (maxlen*width)/height;
	}

	// the scroll position info is either in documentElement or body.
	// why? I don't know. some sites have it in one, others have it
	// in the other. however, this works, so, I don't care enough to
	// find out. I got better things to do.
	var doc = this.browser.docShell.document;
	sleft = doc.documentElement.scrollLeft;
	if (sleft == 0 && doc.body != null)
		sleft = doc.body.scrollLeft;

	stop = doc.documentElement.scrollTop;
	if (stop == 0 && doc.body != null)
		stop = doc.body.scrollTop;

	thumb.width = Math.floor(x);
	thumb.height = Math.floor(y);

	var ctx = thumb.getContext("2d");
	ctx.save();
	ctx.scale(x/width, y/height);

	// we render with a hot pink background so any areas that fail to
	// render show up very obnoxiously and get noticed (and reported).
	// unless, of course, the user visits sites with hot pink backgrounds.
	// meh. I don't want to talk to them anyway.
	ctx.drawWindow(this.browser.contentWindow,
		       sleft, stop, width, height, "rgb(255,0,255)");
	ctx.restore();
},

changeIndex: function (new_index) {
	if (new_index == this.index)
		return;

	var entry;
	if (this.thumbnails[new_index] == null)
		entry = this.thumbnails[new_index] = new historyEntry(this);
	else
		entry = this.thumbnails[new_index];

	if (entry.thumb == null &&
	    this.cache_fill < HistoryThumb.session_max &&
	    this.canvas_cache.length < HistoryThumb.session_max) {
		this.cache_fill += 1;
		entry.thumb = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
		entry.thumb.id = "thumbcanvas" + canvas_counter++;
		entry.thumb.width = HistoryThumb.small_maxwidth;
		entry.thumb.height = 1;
	} else if (entry.thumb == null) {
		entry.thumb = this.canvas_cache.shift();
		if (entry.thumb.dead) {
			delete entry.thumb.dead;
			this.cache_fill += 1;
		}
		entry.thumb.owner_hentry.thumb = null;
		// simple way of keeping pagetext cache memory usage down
		// TODO: put this data on disk to minimize memory usage
		if (HistoryThumb.purge_pagetext)
			entry.thumb.owner_hentry.pagetext = "";
	} else {
		this.canvas_cache.splice(this.canvas_cache.indexOf(entry.thumb), 1);
	}

	entry.thumb.owner_hentry = entry;
	this.canvas_cache.push(entry.thumb);
	this.index = new_index;
},

notifyListeners: function (fh) {
	var i;
	for (i in HistoryThumb.listeners)
		HistoryThumb.listeners[i].thumbnailUpdate(this.browser, fh);
},

// History Listener
OnHistoryGoBack: function (backURI) {
	if (HistoryThumb.session_max > 1)
		this.makeThumbnail();
	this.changeIndex(this.index - 1);
	this.notifyListeners(true);
	return true;
},

OnHistoryGoForward: function (forwardURI) {
	if (HistoryThumb.session_max > 1)
		this.makeThumbnail();
	this.changeIndex(this.index + 1);
	this.notifyListeners(true);
	return true;
},

OnHistoryGotoIndex: function (index, gotoURI) {
	if (HistoryThumb.session_max > 1)
		this.makeThumbnail();
	this.changeIndex(index);
	this.notifyListeners(true);
	return true;
},

OnHistoryNewEntry: function (newURI) {
	if (HistoryThumb.session_max > 1)
		this.makeThumbnail();
	var i;
	for (i = this.index + 1; i < this.thumbnails.length; i++) {
		if (this.thumbnails[i].thumb == null)
			continue;

		this.cache_fill -= 1;
		var canvas = this.thumbnails[i].thumb;
		canvas.dead = true;
		canvas.height = 1;
		canvas.width = HistoryThumb.small_maxwidth;
		this.canvas_cache.splice(this.canvas_cache.indexOf(canvas), 1);
		this.canvas_cache.splice(0, 0, canvas);
	}
	this.thumbnails = this.thumbnails.slice(0, this.index + 1);
	if (this.firstRun != 1)
		this.changeIndex(this.index + 1);
	else
		this.firstRun = 0;

	// this is currently here for sessionsaver. we may extend this in
	// the future to use the given URI in all cases when title is empty
	if (this.thumbnails[this.index].title == null) {
		this.thumbnails[this.index].title = "";
		this.updateEntryURI(this.thumbnails[this.index], newURI);
	}

	this.notifyListeners(true);
},

OnHistoryPurge: function (numEntries) {
	var entry, canvas;
	while (--numEntries >= 0) {
		entry = this.thumbnails.shift();
		canvas = entry.thumb;
		if (canvas != null) {
			this.canvas_cache.splice(this.canvas_cache.indexOf(canvas), 1);
			this.canvas_cache.splice(0, 0, canvas);
		}
		this.index--;
	}
	return true;
},
OnHistoryReload: function (reloadURI, reloadFlags) { return true; },

updateListener: function (e) {
	var tmp = e.currentTarget.thumbcache;
	if (tmp == null)
		return;

	// FIXME: stupid dependence on Reveal behavior
	if (e.currentTarget == getBrowser().selectedBrowser && Reveal.reveal_bc.hidden) {
		var hentry = tmp.thumbnails[tmp.index];
		hentry.title = tmp.browser.contentTitle;
		tmp.updateEntryURI(hentry, tmp.browser.currentURI);
		//tmp.makeThumbnail();
		tmp.updatePagetext();
		tmp.notifyListeners(false);
		return;
	}

	tmp.updatePagetext();

	if (!Reveal.reveal_bc.hidden) {
		tmp.makeThumbnail();
		tmp.notifyListeners(false);
	}

	// we make a guess as to when firefox will finish rendering
	tmp.scheduleRender(250);
}

};

var HistoryThumb = {
	lasttab: null,
	listeners: new Array(),
	tlisteners: new Array(),
	prefs: null,
	small_maxwidth: 0,
	session_max: 0,
	purge_pagetext: true,

	QueryInterface: function (aIID)
	{
	if (aIID.equals(Components.interfaces.nsIPrefBranch2) ||
	    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
	    aIID.equals(Components.interfaces.nsISupports))
		return this;

	throw Components.results.NS_NOINTERFACE;
	},

	observe: function (aSubject, aTopic, aData)
	{
	if (aTopic != "nsPref:changed")
		return;

	switch (aData) {
	case "reveal.thumbnails.small":
		this.small_maxwidth = this.prefs.getIntPref("small");
		if (this.small_maxwidth < 100)
			this.small_maxwidth = 100;
		break;
	case "reveal.thumbnails.session":
		this.session_max = this.prefs.getIntPref("session");
		if (this.session_max < 1)
			this.session_max = 1;
		break;
	case "reveal.thumbnails.cache_pagetext":
		this.cache_pagetext = this.prefs.getBoolPref("cache_pagetext");
		if (document.getElementById("reveal-findmodes").value == "pagetext")
			document.getElementById("reveal-findmodes").value = "url";

		document.getElementById("reveal-pagetext-mode").hidden = !this.cache_pagetext;
		break;
	case "reveal.thumbnails.purge_pagetext":
		this.purge_pagetext = this.prefs.getBoolPref("purge_pagetext");
		break;
	}
	},

	catchNewTab: function(event) {
	if (event.currentTarget != getBrowser().mPanelContainer)
		return;

	(new historyThumbCache(event.target.childNodes[1])).makeThumbnail();

	var i;
	var tabs = getBrowser().mTabs;
	for (i in tabs)
		if (tabs[i].linkedBrowser == null) {
			tabs[i].addEventListener("DOMAttrModified", HistoryThumb.catchTabUpdate, false);
			break;
		}

	for (i in HistoryThumb.tlisteners)
		HistoryThumb.tlisteners[i].addtabUpdate(event.target.childNodes[1]);
	},

	catchDestroyTab: function(event) {
	if(event.currentTarget != getBrowser().mPanelContainer)
		return;

	var b = event.target.childNodes[1];
	if (b.thumbcache.timerID) {
		clearTimeout(b.thumbcache.timerID);
		delete b.thumbcache.timerID;
	}

	var i;
	for (i in HistoryThumb.tlisteners)
		HistoryThumb.tlisteners[i].removetabUpdate(b);
	},

	catchMoveTab: function(event) {
		if (getBrowser().browsers.indexOf(event.target.linkedBrowser) < 0) {
			return;
		}

		var i;
		for (i in HistoryThumb.tlisteners)
			HistoryThumb.tlisteners[i].reordertabUpdate(event.target.childNodes[1]);
	},

	catchTabUpdate: function (event) {
		if (event.target.linkedBrowser == null)
			return;

		var cache = event.target.linkedBrowser.thumbcache;
		var hentry = cache.thumbnails[cache.index];

		if (hentry.box == null)
			return;

		switch (event.attrName) {
		case "busy":
			if (event.newValue == "true")
				cache.locked = true;
			else
				cache.locked = false;
			break;
		case "image":
			if (cache.locked == null || cache.locked == false) {
			if (hentry.src.browser.contentDocument instanceof ImageDocument)
				hentry.icon.setAttribute("src", "");
			else
				hentry.icon.setAttribute("src", event.newValue);
			}
			break;
		case "label":
			hentry.title = hentry.src.browser.contentTitle;
			var i;
			for (i in HistoryThumb.listeners)
				HistoryThumb.listeners[i].thumbnailUpdate(hentry.src.browser, false);
			break;
		}
	},

	updateSelectedCache: function(event) {
	if (HistoryThumb.lasttab == getBrowser().selectedBrowser)
		return;

	var last_cache = HistoryThumb.lasttab.thumbcache;
	if (last_cache == null && HistoryThumb.lasttab != null)
		new historyThumbCache(HistoryThumb.lasttab);

	// avoid trying to render recently nuked tabs
	if (getBrowser().browsers.indexOf(HistoryThumb.lasttab) >= 0) {
		Reveal.closeReveal();
		if (Reveal.lastmode == "history")
			Reveal.toggleHistory();

		last_cache.scheduleRender(0);
	}

	HistoryThumb.lasttab = getBrowser().selectedBrowser;
	},

	addThumbListener: function(listener) {
	this.listeners.push(listener);
	},

	removeThumbListener: function(listener) {
	var tmp = this.listeners.indexOf(listener);
	if (tmp >= 0)
		this.listeners.splice(tmp, 1);
	},

	addTabListener: function(listener) {
	this.tlisteners.push(listener);
	},

	removeTabListener: function(listener) {
	var tmp = this.tlisteners.indexOf(listener);
	if (tmp >= 0)
		this.tlisteners.splice(tmp, 1);
	},

	onLoad: function() {
	this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.thumbnails.");
	this.small_maxwidth = this.prefs.getIntPref("small");
	this.session_max = this.prefs.getIntPref("session");
	this.cache_pagetext = this.prefs.getBoolPref("cache_pagetext");
	this.purge_pagetext = this.prefs.getBoolPref("purge_pagetext");
	document.getElementById("reveal-pagetext-mode").hidden = !this.cache_pagetext;
	if (document.getElementById("reveal-findmodes").value == "pagetext")
		document.getElementById("reveal-findmodes").value = "url";
	Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).addObserver("reveal.thumbnails.", this, false);

	var tmp = getBrowser().mPanelContainer;
	tmp.addEventListener("DOMNodeInserted", this.catchNewTab, false);
	tmp.addEventListener("DOMNodeRemoved", this.catchDestroyTab, false);
	getBrowser().mTabContainer.addEventListener("DOMNodeInserted", this.catchMoveTab, false);

	getBrowser().addEventListener("select", this.updateSelectedCache, false);

	var browsers = getBrowser().browsers;
	var tabs = getBrowser().mTabs;
	if (getBrowser().selectedBrowser == null)
		this.lasttab = browsers[0];
	else
		this.lasttab = getBrowser().selectedBrowser;

	// probably overkill
	for (tmp = browsers.length - 1; tmp >= 0; tmp--) {
		new historyThumbCache(browsers[tmp]);
		tabs[tmp].addEventListener("DOMAttrModified", HistoryThumb.catchTabUpdate, false);
	}
	},

	onUnload: function() {
	var tmp = getBrowser().mPanelContainer;
	tmp.removeEventListener("DOMNodeInserted", this.catchNewTab, false);
	tmp.removeEventListener("DOMNodeRemoved", this.catchDestroyTab, false);

	getBrowser().removeEventListener("select", this.updateSelectedCache, false);
	Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).removeObserver("reveal.thumbnails.", this);
	}
};

var HT_warning = true;
window.addEventListener("load", function(e) { HistoryThumb.onLoad(); }, false);
window.addEventListener("unload", function(e) { HistoryThumb.onUnload(); }, false)
