/*
 * Reveal | previewtip.js
 * Copyright 2005-2006 by Michael Wu
 * Released under GPL v2
 */

var PreviewTip = {
  default_backtext: null,
  default_forwardtext: null,
  current_hentry: null,
  current_text: "",
  preview_index: -1,

  QueryInterface: function (aIID)
  {
	if (aIID.equals(Components.interfaces.nsIPrefBranch2) ||
	    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
	    aIID.equals(Components.interfaces.nsISupports))
		return this;

	throw Components.results.NS_NOINTERFACE;
  },

  load: function() {
	document.getElementById("back-button").removeAttribute("tooltiptext");
	document.getElementById("forward-button").removeAttribute("tooltiptext");
  },

  unload: function() {
	document.getElementById("back-button").setAttribute("tooltiptext", this.default_backtext);
	document.getElementById("forward-button").setAttribute("tooltiptext", this.default_forwardtext);
  },

  onLoad: function() {
	Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).addObserver("reveal.", PreviewTip, false);

	// save standard tooltips
	this.default_backtext = document.getElementById("back-button").getAttribute("tooltiptext");
	this.default_forwardtext = document.getElementById("forward-button").getAttribute("tooltiptext");

	document.getElementById("previewtooltip").addEventListener("popupshowing", PreviewTip.updateTooltip, true);

	var button;
	button = document.getElementById("back-button");
	button.lastChild.addEventListener("mouseover", PreviewTip.setHistoryIndex, false);
	button.lastChild.setAttribute("onpopuphiding", "PreviewTip.removeHistoryIndex(this);");
	button.setAttribute("onmouseover", "PreviewTip.setButtonType(this);");
	button.lastChild.setAttribute("onmouseout", "PreviewTip.removeHistoryIndex(this);");

	button = document.getElementById("forward-button");
	button.lastChild.addEventListener("mouseover", PreviewTip.setHistoryIndex, false);
	button.lastChild.setAttribute("onpopuphiding", "PreviewTip.removeHistoryIndex(this);");
	button.setAttribute("onmouseover", "PreviewTip.setButtonType(this);");
	button.lastChild.setAttribute("onmouseout", "PreviewTip.removeHistoryIndex(this);");

	if (Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.").getBoolPref("bf_preview"))
		this.load();
  },

  onUnload: function() {
	Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).removeObserver("reveal.", this);

	if (Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.").getBoolPref("bf_preview"))
		this.unload();
  },

  observe: function (aSubject, aTopic, aData) {
	if (aTopic != "nsPref:changed")
		return;

	switch (aData) {
	case "reveal.bf_preview":
		if (Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.").getBoolPref("bf_preview"))
			this.load();
		else
			this.unload();
		break;
	}
  },

  setHistoryIndex: function (event) {
	var target = event.target;
	if (target.nodeName == "menuitem")
		PreviewTip.preview_index = Number(target.getAttribute("index"));

	if (isNaN(PreviewTip.preview_index))
		PreviewTip.preview_index = -1;
  },

  setButtonType: function (target) {
	var cache = getBrowser().selectedBrowser.thumbcache;
	if (target.id == "back-button") {
		PreviewTip.current_hentry = cache.thumbnails[cache.index - 1];
		PreviewTip.current_text = PreviewTip.default_backtext;
	} else if (target.id == "forward-button") {
		PreviewTip.current_hentry = cache.thumbnails[cache.index + 1];
		PreviewTip.current_text = PreviewTip.default_forwardtext;
	}
  },

  removeHistoryIndex: function (target) {
	if (target.nodeName == "menupopup")
		PreviewTip.preview_index = -1;
  },

  updateTooltip: function(event) {
	var hentry, target;
	target = event.target;
	if (PreviewTip.preview_index >= 0)
		hentry = getBrowser().selectedBrowser.thumbcache.thumbnails[PreviewTip.preview_index];
	else
		hentry = PreviewTip.current_hentry;

	var canvas = target.firstChild.firstChild;
	var label = target.firstChild.lastChild;

	canvas.style.display = "none";
	label.style.width = label.style.maxWidth = "";

	target.setAttribute("noautohide", "true");
	if (hentry == null || hentry.thumb == null) {
		label.value = PreviewTip.current_text;
		label.style.display = "-moz-box";
		target.setAttribute("noautohide", "false");
	} else if (PreviewTip.preview_index < 0) {
		label.value = hentry.title;
		label.style.display = "-moz-box";
	} else
		label.style.display = "none";

	setTimeout(PreviewTip.render, 0, label, canvas, hentry);
  },

  render: function (label, canvas, hentry) {
	if (hentry != null && hentry.thumb != null) {
		canvas.style.display = "block";
		label.style.width = label.style.maxWidth =
					HistoryThumb.small_maxwidth + "px";
		canvas.style.height = hentry.thumb.height + "px";
		canvas.style.width = hentry.thumb.width + "px";
		canvas.height = hentry.thumb.height;
		canvas.width = hentry.thumb.width;
		canvas.getContext("2d").drawImage(hentry.thumb, 0, 0);
	} else
		canvas.style.display = "none";
  }
};

window.addEventListener("load", function(e) { PreviewTip.onLoad(); }, false); 
window.addEventListener("unload", function(e) { PreviewTip.onUnload(); }, false); 
