/*
 * Reveal | quickmag.js
 * Copyright 2005-2006 by Michael Wu
 * Released under GPL v2
 */

var QuickMag = {
	magOn: false,
	boxWidth: 200,
	mag_factor: 1.2,
	mag_count: 1,
	canvas: null,
	magbox: null,
	rx: 0,
	ry: 0,
	scaled_width: 0,
	scalemode: false,

	QueryInterface: function (aIID)
	{
		if (aIID.equals(Components.interfaces.nsIPrefBranch2) ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		    aIID.equals(Components.interfaces.nsISupports))
			return this;

		throw Components.results.NS_NOINTERFACE;
	},

	observe: function (aSubject, aTopic, aData)
	{
		if (aTopic != "nsPref:changed")
			return;

		switch (aData) {
		case "reveal.quickmag":
			if (getBoolPref("reveal.quickmag"))
				getBrowser().mPanelContainer.addEventListener("click", this.toggleMag, false);
			else
				getBrowser().mPanelContainer.removeEventListener("click", this.toggleMag, false);
			break;
		case "reveal.quickmag_size":
			break;
			this.boxWidth = this.prefs.getIntPref("quickmag_size");
			this.canvas.width = this.canvas.height = this.boxWidth;
			this.scaled_width = Math.ceil(this.boxWidth/this.mag_factor) + 1;

			var ctx = this.canvas.getContext("2d");
			ctx.save();
			ctx.scale(this.mag_factor, this.mag_factor);
			break;
		}
	},

	calcMag: function (x, y) {
		var b = getBrowser().selectedBrowser;
		var doc = b.contentDocument;
		this.rx = this.ry = 0;

		x -= b.boxObject.x;
		y -= b.boxObject.y;

		if (doc.documentElement != null && doc.documentElement.scrollLeft != null) {
			this.rx = doc.documentElement.scrollLeft;
			this.ry = doc.documentElement.scrollTop;
		}

		if (this.rx == 0 && this.ry == 0 && doc.body != null) {
			this.rx = doc.body.scrollLeft;
			this.ry = doc.body.scrollTop;
		}

		this.rx += x;
		this.ry += y;
	},

	renderMag: function () {
		var ctx = this.canvas.getContext("2d");

		ctx.drawWindow(getBrowser().selectedBrowser.contentWindow, this.rx - (this.scaled_width/2), this.ry - (this.scaled_width/2), this.scaled_width, this.scaled_width, "rgb(0,0,0)");
	},

	handleWheel: function (e) {
		QuickMag.adjustZoom(QuickMag.scalemode, e.detail > 0);
	},

	adjustZoom: function (scale, up) {
		var ctx = QuickMag.canvas.getContext("2d");

		if (scale) {
			if (up && QuickMag.boxWidth < 500) {
				QuickMag.boxWidth += 50;
				QuickMag.left -= 25;
				QuickMag.top -= 25;
			} else if (!up && QuickMag.boxWidth > 100) {
				QuickMag.boxWidth -= 50;
				QuickMag.left += 25;
				QuickMag.top += 25;
			} else return;

			QuickMag.canvas.width = QuickMag.canvas.height = QuickMag.boxWidth;
			QuickMag.magbox.style.left = QuickMag.left + "px";
			QuickMag.magbox.style.top = QuickMag.top + "px";
			QuickMag.prefs.setIntPref("quickmag_size", QuickMag.boxWidth);
		} else {
			if (up && QuickMag.mag_count <= 10) {
				QuickMag.mag_factor *= 1.3;
				QuickMag.mag_count += 1;
			} else if (!up && QuickMag.mag_count > 1) {
				QuickMag.mag_factor /= 1.3;
				QuickMag.mag_count -= 1;
			} else return;

			ctx.restore();
		}

		QuickMag.scaled_width = Math.ceil(QuickMag.boxWidth/QuickMag.mag_factor) + 1;
		ctx.save();
		ctx.scale(QuickMag.mag_factor, QuickMag.mag_factor);
		QuickMag.renderMag();
	},

	handleKeys: function (e) {
		switch (e.keyCode) {
		case e.DOM_VK_PAGE_UP:
			QuickMag.adjustZoom(false, false);
			break;
		case e.DOM_VK_PAGE_DOWN:
			QuickMag.adjustZoom(false, true);
			break;
		case e.DOM_VK_HOME:
			QuickMag.adjustZoom(true, false);
			break;
		case e.DOM_VK_END:
			QuickMag.adjustZoom(true, true);
			break;
		}
	},

	trackMouse: function (e) {
		if (!(e.target instanceof HTMLCanvasElement))
			return;

		var half_width = QuickMag.boxWidth/2;
		var boxObj = e.currentTarget.boxObject;
		var left = e.clientX - half_width;
		var top = e.clientY - half_width;
		QuickMag.left = left;
		QuickMag.top = top;
		QuickMag.magbox.style.left = left + "px";
		QuickMag.magbox.style.top = top + "px";
		QuickMag.calcMag(e.clientX, e.clientY);
		QuickMag.renderMag();
	},

	disableMag: function (e) {
		if (e != null && e.button == 2)
			return;

		if (!QuickMag.magOn)
			return;

		QuickMag.magOn = false;
		QuickMag.magbox.hidden = true;
		window.removeEventListener("mousemove", QuickMag.trackMouse, false);
	},

	enableScale: function (e) {
		if (e.button == 2)
			QuickMag.scalemode = true;

		QuickMag.magbox.style.borderColor = "white";
	},

	disableScale: function (e) {
		if (e.button == 2)
			QuickMag.scalemode = false;

		QuickMag.magbox.style.borderColor = "";
	},

	toggleMag: function (e) {
		var good = false;
		if (getBrowser().selectedBrowser.contentDocument instanceof ImageDocument) {
			if (e.detail == 2 && e.button == 0)
				good = true;
		} else {
			if (e.detail == 3 && e.button == 0)
				good = true;

			if (e.detail == 2 && e.button == 2)
				good = true;
		}

		if (!good)
			return;

		if (QuickMag.magOn)
			QuickMag.disableMag(null);
		else if (e.currentTarget == getBrowser().mPanelContainer) {
			var half_width = QuickMag.boxWidth/2;
			var boxObj = e.currentTarget.boxObject;
			QuickMag.magbox.addEventListener("click", QuickMag.disableMag, false);
			QuickMag.magbox.addEventListener("mousedown", QuickMag.enableScale, false);
			QuickMag.magbox.addEventListener("mouseup", QuickMag.disableScale, false);
			QuickMag.magbox.addEventListener("DOMMouseScroll", QuickMag.handleWheel, false);
			QuickMag.left = boxObj.x + e.clientX - half_width;
			QuickMag.top = boxObj.y + e.clientY - half_width;
			QuickMag.magbox.style.left = QuickMag.left + "px";
			QuickMag.magbox.style.top = QuickMag.top + "px";
			QuickMag.magbox.hidden = false;
			window.addEventListener("mousemove", QuickMag.trackMouse, false);
			QuickMag.calcMag(boxObj.x + e.clientX, boxObj.y + e.clientY);
			QuickMag.renderMag();
			QuickMag.magOn = !QuickMag.magOn;
			document.getElementById("quickmag-input").focus();
		}
	},

	onLoad: function () {
		Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).addObserver("reveal.", QuickMag, false);

		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.");

		this.boxWidth = this.prefs.getIntPref("quickmag_size");

		this.canvas = document.getElementById("quickmag-canvas");
		this.canvas.width = this.canvas.height = this.boxWidth;
		this.scaled_width = Math.ceil(this.boxWidth/this.mag_factor) + 1;

		document.getElementById("quickmag-input").addEventListener("keypress", QuickMag.handleKeys, true);
		var ctx = this.canvas.getContext("2d");
		ctx.save();
		ctx.scale(this.mag_factor, this.mag_factor);

		this.magbox = document.getElementById("quickmag-box");
		this.magbox.style.left = this.magbox.style.top = "0px";
		if (getBoolPref("reveal.quickmag"))
			getBrowser().mPanelContainer.addEventListener("click", this.toggleMag, false);
	},

	onUnload: function () {
		if (getBoolPref("reveal.quickmag"))
			getBrowser().mPanelContainer.removeEventListener("click", this.toggleMag, false);
	}
};

window.addEventListener("load", function (e) { QuickMag.onLoad(); }, false);
window.addEventListener("unload", function (e) { QuickMag.onUnload(); }, false);
