/*
 * Reveal | reveal.js
 * Copyright 2005-2006 by Michael Wu
 * Released under GPL v2
 */

var Reveal = {
	tabthumbs: new Array(),
	current_thumbs: null,
	mwindow: null,
	bbar: null,
	hscroll: 0,
	maxhscroll: 0,
	reveal_bc: null,
	stack: true,
	current_tab: null,
	prefs: null,
	spiff: false,
	findmode: "url",
	findbar: null,
	speedmult: 0.2,
	regex: false,
	match_case: false,
	selected: null,
	dragging: false,
	dragTarget: null,
	animID: -1,
	lastmode: "tab",
	findpattern: null,
	red: 0,
	green: 0,
	blue: 0,

	QueryInterface: function (aIID)
	{
		if (aIID.equals(Components.interfaces.nsIPrefBranch2) ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		    aIID.equals(Components.interfaces.nsISupports))
			return this;

		throw Components.results.NS_NOINTERFACE;
	},

	// takes hsv values between 0 and 1
	HSVtoRGB: function (h, s, v) {
		if (s == 0) {
			var tmp = Math.floor(v*255);
			this.red = tmp;
			this.green = tmp;
			this.blue = tmp;
			return;
		}

		var r, g, b;
		var i, f, p, q, t;

		h *= 6;
		if (h == 6)
			h = 0;

		i = Math.floor(h);
		f = h - i;
		p = v*(1 - s);
		q = v*(1 - s*f);
		t = v*(1 - s*(1 - f));

		switch (i) {
		case 0:
			r = v;
			g = t;
			b = p;
			break;
		case 1:
			r = q;
			g = v;
			b = p;
			break;
		case 2:
			r = p;
			g = v;
			b = t;
			break;
		case 3:
			r = p;
			g = q;
			b = v;
			break;
		case 4:
			r = t;
			g = p;
			b = v;
			break;
		default:
			r = v;
			g = p;
			b = q;
			break;
		}

		this.red = Math.floor(r*255);
		this.green = Math.floor(g*255);
		this.blue = Math.floor(b*255);
	},

	closeReveal: function ()
	{
		if (this.reveal_bc.hidden)
			return;

		this.reveal_bc.hidden = true;
		var findhbox = document.getElementById("reveal-find-textbox-decor").parentNode;
		findhbox.removeChild(this.finddrop);
		findhbox.removeChild(this.modedrop);
		var browser = getBrowser().selectedBrowser;
		if (browser.currentURI != null &&
		    browser.currentURI.spec == "about:blank")
			document.getElementById("urlbar").focus();
		else
			browser.contentWindow.focus();

		var findbox = document.getElementById("reveal-find-textbox");
		if (findbox.value != "") {	// reset search
			findbox.value = "";
			this.findpattern = null;

			var i;
			for (i in this.current_thumbs)
				this.current_thumbs[i].style.display = "-moz-box";
		}

		findbox.style.backgroundColor = "";
		findbox.style.color = "";

	},

	previewClick: function (current, event)
	{
		if (current == this.dragTarget) {
			this.dragTarget = null;
			return;
		}

		var b = current.owner_hentry.src.browser;
		var blist = getBrowser().browsers;
		var idx = current.owner_hentry.src.thumbnails.indexOf(current.owner_hentry);
		var i = blist.indexOf(b);
		if (i < 0)
			return;

		if (this.current_thumbs == this.tabthumbs &&
		    event instanceof MouseEvent && event.button == 2)
			return;

		if (event instanceof MouseEvent && event.button == 1) {
			if (this.current_thumbs == this.tabthumbs) {
			        var j;
		        	var tabs = getBrowser().mTabs;
			        for (j in tabs) {
					if (tabs[j].linkedBrowser == current.owner_hentry.src.browser) {
						getBrowser().removeTab(tabs[j]);
						break;
					}
				}
			} else
				getBrowser().selectedTab = getBrowser().addTab(current.owner_hentry.uri);
		} else if (event instanceof KeyboardEvent && event.ctrlKey)
			getBrowser().selectedTab = getBrowser().addTab(current.owner_hentry.uri);
		else {
			if (idx != current.owner_hentry.src.index)
				current.owner_hentry.src.browser.gotoIndex(idx);

			getBrowser().mTabBox.selectedIndex = i;
			this.closeReveal();
		}
	},

	advanceFrame: function (self)
	{
		self.animID = -1;
		if (self.reveal_bc.hidden)
			return;

		var i, cont;
		var target_top, target_left, newtop, newleft, speedmult;
		var deltaX, deltaY;
		cont = false;
		for (i in self.current_thumbs) {
			var tmp = self.current_thumbs[i];
			if (tmp == self.dragTarget && tmp.dragLeft != null) {
				target_left = tmp.dragLeft;
				target_top = tmp.dragTop;
				speedmult = self.speedmult*3;
			} else {
				target_left = tmp.target_left + self.hscroll;
				target_top = tmp.target_top;
				speedmult = self.speedmult;
			}

			if (tmp.current_left == null || isNaN(tmp.current_left))
				tmp.current_left = target_left;
			if (tmp.current_top == null || isNaN(tmp.current_top))
				tmp.current_top = target_top;

			deltaX = target_left - tmp.current_left;
			deltaY = target_top - tmp.current_top;
			newleft = tmp.current_left + (deltaX*speedmult);
			newtop = tmp.current_top + (deltaY*speedmult);

			if (!self.spiff ||
			    (Math.abs(deltaX) + Math.abs(deltaY) < 1) ) {
				tmp.style.left = target_left + "px";
				tmp.style.top = target_top + "px";
				tmp.current_left = target_left;
				tmp.current_top = target_top;
				continue;
			}

			cont = true;
			tmp.style.left = Math.round(newleft) + "px";
			tmp.style.top = Math.round(newtop) + "px";
			tmp.current_left = newleft;
			tmp.current_top = newtop;
		}

		if (cont && self.animID < 0)
			self.animID = setTimeout(self.advanceFrame, 40, self);
	},

	startAnimation: function ()
	{
		if (this.animID >= 0)
			return;

		if (this.spiff)		// delay seems to improve animation
			this.animID = setTimeout(this.advanceFrame, 10, this);
		else
			this.advanceFrame(this);
	},

	positionPreviews: function ()
	{
		if (this.reveal_bc == null)
			return;

		if (this.reveal_bc.hidden == true || this.current_thumbs.length == 0)
			return;

		var hcounter = 10;
		var center = Math.floor(this.mwindow.boxObject.height/2);

		var i;
		var up = true;
		for (i in this.current_thumbs) {
			var tmp = this.current_thumbs[i];
			if (tmp.style.display == "none") {
				tmp.style.left = "0px";
				tmp.style.top = "0px";
				continue;
			}

			var old_left, old_top;
			old_left = tmp.current_left;
			old_top = tmp.current_top;

			var objwidth = tmp.firstChild.boxObject.width + 4;

			if (this.stack) {
				if (hcounter > objwidth/2)
					hcounter -= objwidth/2;

				tmp.target_left = hcounter;
				if (up)
					tmp.target_top = center - tmp.boxObject.height - 5;
				else
					tmp.target_top = center + 5;

				up = !up;
			} else {
				tmp.target_left = hcounter;
				tmp.target_top = center - tmp.boxObject.height/2;
			}

			if (old_top == null || isNaN(old_top)) {
				old_top = tmp.target_top;
				if (tmp.target_left < (this.mwindow.boxObject.width/2))
					old_left = -objwidth;
				else
					old_left = this.mwindow.boxObject.width;
			}

			if (tmp != this.dragTarget) {
				tmp.style.left = old_left + "px";
				tmp.style.top = old_top + "px";
				tmp.current_left = old_left;
				tmp.current_top = old_top;
			}

			if (this.stack)
				hcounter += 5;
			else
				hcounter += 10;

			hcounter += objwidth;
		}

		this.maxhscroll = hcounter - this.mwindow.boxObject.width;
		if (this.maxhscroll < 0)
			this.maxhscroll = 0;

		if (this.hscroll > 0)
			this.hscroll = 0;
		else if (this.hscroll < -this.maxhscroll)
			this.hscroll = -this.maxhscroll;

		this.showSelected();
		this.startAnimation();
	},

	scrollPreviews: function (delta)
	{
		this.hscroll += delta;
		this.startAnimation();
	},

	openTabMenu: function (target, event)
	{
		if (Reveal.tabthumbs != Reveal.current_thumbs)
			return;

		var menu = getBrowser().boxObject.lastChild.getElementsByAttribute("anonid", "tabContextMenu")[0];
		var tabs = getBrowser().mTabs;
		getBrowser().updatePopupMenu(menu);
		var i;
		for (i in tabs)
			if (tabs[i].linkedBrowser == target.owner_hentry.src.browser) {
				// BUG: if user right click drags outside the preview. the next regular click on it will be completely ignored..
				Reveal.dragTarget = Reveal.selected = target;
				Reveal.showSelected();
				menu.style.visibility = "visible";
				menu.showPopup(target, event.screenX, event.screenY, "context","","");
				getBrowser().mContextTab = tabs[i];
				break;
			}
	},

	dragPreviewStart: function (target, event)
	{
		if (Reveal.current_thumbs != Reveal.tabthumbs)
			return;

		target.style.cursor = "-moz-grabbing";
		target.style.zIndex = "1000";
		target.dragX = event.screenX;
		target.dragY = event.screenY;
		target.dragLeft = target.current_left;
		target.dragTop = target.current_top;
		Reveal.dragTarget = Reveal.selected = target;
		Reveal.showSelected();
		window.addEventListener("mousemove", Reveal.dragPreview, true);
		window.addEventListener("mouseup", Reveal.dragPreviewEnd, true);

		event.stopPropagation();
	},

	dragPreviewEnd: function (event)
	{
		window.removeEventListener("mousemove", Reveal.dragPreview, true);
		window.removeEventListener("mouseup", Reveal.dragPreviewEnd, true);
		Reveal.dragTarget.style.cursor = "";
		Reveal.dragTarget.style.zIndex = "";
		delete Reveal.dragTarget.dragLeft;
		delete Reveal.dragTarget.dragTop;
		Reveal.syncCurrentThumbs(Reveal.current_thumbs);

		var tabs = getBrowser().mTabs;
		var i;
		for (i in tabs) {
			var newidx = Reveal.current_thumbs.indexOf(Reveal.dragTarget);
			if (tabs[i].linkedBrowser == Reveal.dragTarget.owner_hentry.src.browser) {
				if (i == newidx) // no real change
					break;

				getBrowser().moveTabTo(tabs[i], newidx);
				return;
			}
		}

		Reveal.startAnimation();
	},

	dragPreview: function (event)
	{
		var target = Reveal.dragTarget;
		target.dragLeft += event.screenX - target.dragX;
		target.dragTop += event.screenY - target.dragY;
		target.dragX = event.screenX;
		target.dragY = event.screenY;
		if (Reveal.animID < 0) {
			target.current_left = target.dragLeft;
			target.current_top = target.dragTop;
			target.style.left = target.current_left + "px";
			target.style.top = target.current_top + "px";
		}

		var i;
		for (i = Reveal.current_thumbs.length - 1; i >= 0; i--) {
			if (Reveal.current_thumbs[i].target_left + Reveal.hscroll < target.current_left && (i >= (Reveal.current_thumbs.length - 1) || (Reveal.current_thumbs[i+1].target_left + Reveal.hscroll) >= target.current_left)) {
				var old_index = Reveal.current_thumbs.indexOf(target);

				if (i != old_index) {
					Reveal.current_thumbs.splice(old_index, 1);
					Reveal.current_thumbs.splice(i, 0, target);
					Reveal.syncCurrentThumbs(Reveal.current_thumbs);
					return;
				}
			}
		}
		Reveal.startAnimation();
	},

	dragBackbar: function (event)
	{
		if (!Reveal.dragging) {
			Reveal.mwindow.removeEventListener("mousemove", Reveal.dragBackbar, true);
			return;
		}

		var diff = event.screenX - Reveal.grab_x;

		if (diff > 0) {
			if ((Reveal.hscroll + diff) < 0)
				Reveal.scrollPreviews(diff);
		} else {
			if ((Reveal.hscroll + diff) > -Reveal.maxhscroll)
				Reveal.scrollPreviews(diff);
		}

		Reveal.grab_x = event.screenX;
	},

	deactivateDrag: function (event) {
		Reveal.dragging = false;
		Reveal.bbar.style.cursor = "";
		Reveal.mwindow.removeEventListener("mouseup", Reveal.deactivateDrag, true);
	},

	showSelected: function ()
	{
		var i;
		for (i in this.current_thumbs) {
			if (this.current_thumbs[i] == this.selected)
				this.current_thumbs[i].setAttribute("selected", "");
			else
				this.current_thumbs[i].removeAttribute("selected");
		}

		if (this.selected == null ||
		    (this.dragTarget != null && this.dragTarget.dragLeft != null))
			return;

		if ((this.selected.target_left + this.hscroll) < 0) {
			this.hscroll = -(this.selected.target_left - 5);
			this.startAnimation();
		} else if ((this.selected.target_left + this.selected.firstChild.boxObject.width + 4 + this.hscroll) >
			   this.mwindow.boxObject.width) {
			this.hscroll = this.mwindow.boxObject.width - (this.selected.target_left + this.selected.firstChild.boxObject.width + 11);
			this.startAnimation();
		}
	},

	distColors: function ()
	{
		var host_list = new Array();
		var i, j, hostname, length;

		length = 0;
		this.findCurrentTab();
		for (i in this.current_thumbs) {
			hostname = this.current_thumbs[i].owner_hentry.host;
			if (host_list[hostname] == null) {
				host_list[hostname] = this;
				length += 1;
			}

			if (this.current_thumbs[i] == this.current_tab)
				this.current_thumbs[i].owner_hentry.label.setAttribute("current", "");
			else
				this.current_thumbs[i].owner_hentry.label.removeAttribute("current");
		}

		j = 0;
		for (i in host_list)
			host_list[i] = j++/length;

		for (i in this.current_thumbs) {
			this.HSVtoRGB(host_list[this.current_thumbs[i].owner_hentry.host], .6, 1);
			this.current_thumbs[i].style.backgroundColor = 
				"rgb(" + this.red + "," + this.green + "," + this.blue + ")";
		}
	},

	generateTNElement: function (browser)
	{
		var ctx, cache, src;
		cache = browser.thumbcache;
		if (cache == null || cache.index < 0)
			return;

		var index = getBrowser().browsers.indexOf(browser);
		if (index < 0)
			index = this.tabthumbs.length;
		var hentry = cache.thumbnails[cache.index];
		var redist = true;
		var canvas = hentry.thumb;
		var box = hentry.box;
		var label = hentry.label;
		var container = hentry.container;
		var icon = hentry.icon;

		if (box == null) {
			this.findCurrentTab();
			hentry.box = box = document.createElement("vbox");
			hentry.label = label = document.createElement("label");
			box.setAttribute("class", "reveal-tnbox");
			box.setAttribute("onclick", "Reveal.previewClick(this, event);");
			box.setAttribute("oncontextmenu", "Reveal.openTabMenu(this, event);");
			box.setAttribute("ondraggesture", "Reveal.dragPreviewStart(this, event);");
			hentry.container = container = document.createElement("vbox");
			container.style.MozBoxAlign = "center";
			label.setAttribute("crop", "end");
			label.setAttribute("class", "reveal-label");
			container.appendChild(label);
			box.appendChild(container);
			if (this.current_tab != null) {
				box.target_left = this.current_tab.target_left;
				box.target_top = this.current_tab.target_top;
			}

			var img = document.createElementNS("http://www.w3.org/1999/xhtml", "img");;
			hentry.icon = icon = img;
			img.setAttribute("class", "reveal-img");
			//if (cache.locked == true)
			//	img.setAttribute("src", "chrome://global/skin/throbber/Throbber-small.gif");
			container.appendChild(img);

			// TODO: add DnD support
			//box.addEventListener("dragover", function(e) { debugger; }, true);
			//box.addEventListener("dragdrop", function(e) { debugger; return true; }, true);
		}

		if (canvas != null && container.firstChild != canvas) {
			container.insertBefore(canvas, container.firstChild);
			box.owner_hentry = canvas.owner_hentry;
		}

		if (hentry.title == "")
			label.setAttribute("value", hentry.uri);
		else
			label.setAttribute("value", hentry.title);

		label.setAttribute("tooltiptext", label.getAttribute("value"));
		box.style.maxWidth = label.style.width = label.style.maxWidth = HistoryThumb.small_maxwidth + "px";
		icon.style.left = (HistoryThumb.small_maxwidth - 17) + "px";

		if (this.tabthumbs[index] != box && this.tabthumbs[index] != null) {
			if (this.tabthumbs[index] == this.selected)
				this.selected = box;

			if (this.tabthumbs[index].owner_hentry.host ==
				box.owner_hentry.host ||
				this.tabthumbs != this.current_thumbs)
				redist = false;
		}

		var old = this.tabthumbs[index];
		this.tabthumbs[index] = box;
		if (this.tabthumbs == this.current_thumbs &&
		    (this.dragTarget == null || this.dragTarget.dragLeft == null)) {
			if (old == null)
				this.bbar.appendChild(box);
			else
				this.bbar.replaceChild(box, old);

		} else if (this.lastmode == "all")
			this.syncCurrentThumbs(this.getAllArray());

		if (redist)
			this.distColors();

		if (this.checkMatch(hentry))
			box.style.display = "-moz-box";
		else
			box.style.display = "none";
	},

	findCurrentTab: function () {
		var cache = getBrowser().selectedBrowser.thumbcache;
		if (cache == null || cache.index < 0) {
			this.current_tab = null;
			return;
		}

		this.current_tab = cache.thumbnails[cache.index].box;
	},

	toggle: function ()
	{
		if (!this.reveal_bc.hidden)
			this.closeReveal();
		else {
			getBrowser().selectedBrowser.thumbcache.makeThumbnail();
			this.findCurrentTab();
			if (this.current_thumbs.indexOf(this.current_tab) > -1)
				this.selected = this.current_tab;

			this.distColors();
			this.reveal_bc.hidden = false;

			// this fixes problems on Windows
			var findhbox = document.getElementById("reveal-find-textbox-decor").parentNode;
			var findcase = document.getElementById("reveal-find-case");
			findhbox.insertBefore(this.finddrop, findcase);
			findhbox.insertBefore(this.modedrop, findcase);
			this.modedrop.style.display = "-moz-box";
			document.getElementById("reveal-findmodes").style.display = "-moz-box";
			document.getElementById("reveal-pagetext-mode").hidden = !HistoryThumb.cache_pagetext;

			this.positionPreviews();
			document.getElementById("reveal-find-textbox").focus();
		}
	},

	openReveal: function () {
		if (this.reveal_bc.hidden)
			this.toggle();
		else
			document.getElementById("reveal-find-textbox").focus();
	},

	checkMatch: function (hentry)
	{
	if (this.findpattern == null)
		return true;

	var cached = true;
	if (hentry.src.thumbnails.indexOf(hentry) ==
	    hentry.src.index)
		cached = false;

	switch (this.findmode) {
	case "url":
		if (cached)
			return (hentry.uri.search(this.findpattern) >= 0);
		else
			return (hentry.src.browser.currentURI.spec.search(this.findpattern) >= 0);
	case "pagetext":
		return (hentry.pagetext.search(this.findpattern) >= 0);
	case "title":
		if (cached)
			return (hentry.title.search(this.findpattern) >= 0);
		else
			return (hentry.src.browser.contentTitle.search(this.findpattern) >= 0);
	case "any":
		if (cached) {
			if (hentry.uri.search(this.findpattern) >= 0 ||
			    hentry.title.search(this.findpattern) >= 0)
				return true;
		} else {
			if (hentry.src.browser.currentURI.spec.search(this.findpattern) >= 0 ||
			    hentry.src.browser.contentTitle.search(this.findpattern) >= 0)
				return true;
		}

		if (hentry.pagetext.search(this.findpattern) >= 0)
			return true;

		return false;
	}

	// should never happen..
	return true;
	},

	find: function (src)
	{
		var pattern;

		pattern = src.value;
		if (!this.regex)
			pattern = pattern.replace(/[\\\^\$\*\+\?\.\(\)\:\=\|\{\}\[\]\,]/g, "\\$&");

		try {
			if (this.match_case)
				this.findpattern = new RegExp(pattern);
			else
				this.findpattern = new RegExp(pattern, "i");
		} catch (e) {
			src.style.backgroundColor = "#FF6666";
			src.style.color = "white";
			return;
		}

		src.style.backgroundColor = "";
		src.style.color = "";

		if (this.selected != null)
			this.selected = null;

		var i, thumb;
		for (i in this.current_thumbs) {
			thumb = this.current_thumbs[i];
			if (src.value == "" ||
			    this.checkMatch(thumb.owner_hentry))
				thumb.style.display = "-moz-box";
			else
				thumb.style.display = "none";
		}

		if (src.value == "")
			this.findpattern = null;

		this.selectPreview(true);
		this.positionPreviews();
	},

	updateFindMode: function (src)
	{
		switch (src.id) {
		case "reveal-findmodes":
			this.findmode = src.value;
			break;
		case "reveal-find-regex":
			this.regex = src.checked;
			break;
		case "reveal-find-case":
			this.match_case = src.checked;
			break;
		}

		this.find(document.getElementById("reveal-find-textbox"));
		document.getElementById("reveal-find-textbox").focus();
	},

	updateMode: function (src)
	{
		if (src.value == Reveal.lastmode)
			return;

		switch (src.value) {
		case "tab":
			Reveal.syncCurrentThumbs(Reveal.tabthumbs);
			break;
		case "history":
			Reveal.toggleHistory();
			break;
		case "all":
			Reveal.syncCurrentThumbs(Reveal.getAllArray());
			break;
		}

		Reveal.lastmode = src.value;
		document.getElementById("reveal-find-textbox").focus();
	},

	selectPreview: function (forward) {
		if (Reveal.current_thumbs.length < 1)
			return;

		var i = Reveal.current_thumbs.indexOf(Reveal.selected);
		var j;
		var inc;
		if (forward)
			inc = 1;
		else
			inc = Reveal.current_thumbs.length - 1;

		for (j in Reveal.current_thumbs) {
			i += inc;
			i %= Reveal.current_thumbs.length;
			if (Reveal.current_thumbs[i].style.display != "none") {
				Reveal.selected = Reveal.current_thumbs[i];
				Reveal.showSelected();
				break;
			}
		}
	},

	syncCurrentThumbs: function (thumbs) {
		var i;
		for (i = this.bbar.childNodes.length - 1; i >= 0; i--)
			if (this.bbar.childNodes[i].owner_hentry != null)
				this.bbar.removeChild(this.bbar.childNodes[i]);

		if (thumbs.indexOf(this.selected) < 0 &&
		    thumbs.indexOf(this.current_tab) > -1)
			this.selected = this.current_tab;

		for (i in thumbs) {
			this.bbar.appendChild(thumbs[i]);
			thumbs[i].hidden = false;
			if (this.checkMatch(thumbs[i].owner_hentry))
				thumbs[i].style.display = "-moz-box";
			else
				thumbs[i].style.display = "none";
		}

		this.current_thumbs = thumbs;
		if (this.dragTarget == null || this.dragTarget.dragLeft == null)
			this.distColors();
		this.positionPreviews();
	},

	getHistoryArray: function () {
		var thumbs = new Array();
		var tn = this.selected.owner_hentry.src.thumbnails;
		var i;
		for (i in tn)
			thumbs.push(tn[i].box);

		return thumbs;
	},

	getTabArray: function () {
		var thumbs = new Array();
		var tb = getBrowser().mTabs;
		var i, j;
		for (i = 0; i < tb.length; i++) {
			j = null;
			// yea, what the hell
			if (tb[i].linkedBrowser != null)
				j = tb[i].linkedBrowser.thumbcache;

			if (j == null || j.index < 0)
				continue;

			thumbs.push(j.thumbnails[j.index].box);
		}

		return thumbs;
	},

	getAllArray: function () {
		var thumbs = new Array();
		var tb = getBrowser().mTabs;
		var i, j, k;
		for (i = 0; i < tb.length; i++) {
			j = null;
			if (tb[i].linkedBrowser != null)
				j = tb[i].linkedBrowser.thumbcache;

			if (j == null || j.index < 0)
				continue;

			for (k in j.thumbnails)
				thumbs.push(j.thumbnails[k].box);
		}

		return thumbs;
	},

	toggleHistory: function () {
		var thumbs;
		if (this.lastmode == "history") {
			thumbs = this.tabthumbs;
			this.lastmode = "tab";
			this.modedrop.value = "tab";
		} else if (this.selected != null) {
			thumbs = this.getHistoryArray();
			this.lastmode = "history";
			this.modedrop.value = "history";
		} else {
			this.modedrop.value = "tab";
			return;
		}

		this.syncCurrentThumbs(thumbs);
	},

	toggleAll: function () {
		var thumbs;
		if (this.lastmode != "all") {
			thumbs = this.getAllArray();
			this.lastmode = "all";
			this.modedrop.value = "all";
		} else {
			thumbs = this.tabthumbs;
			this.lastmode = "tab";
			this.modedrop.value = "tab";
		}

		this.syncCurrentThumbs(thumbs);
	},

	pagePreviews: function (forward) {
		if (Reveal.current_thumbs.length < 1)
			return;

		var i = Reveal.current_thumbs.indexOf(Reveal.selected);
		var j, count;
		var inc;
		if (forward)
			inc = 1;
		else
			inc = Reveal.current_thumbs.length - 1;

		for (j in Reveal.current_thumbs) {
			var current = Reveal.current_thumbs[i];

			if (forward) {
				if (i == Reveal.current_thumbs.length - 1) {
					Reveal.selected = current;
					Reveal.showSelected();
				}
			} else {
				if (i == 0) {
					Reveal.selected = current;
					Reveal.showSelected();
				}
			}

			i += inc;
			i %= Reveal.current_thumbs.length;

			if (forward) {
				if ((current.target_left + Reveal.hscroll + current.boxObject.width) <
				    Reveal.mwindow.boxObject.width)
					continue;

				Reveal.hscroll -= Reveal.mwindow.boxObject.width;
			} else {
				if ((current.target_left + Reveal.hscroll) > 0)
					continue;

				Reveal.hscroll += Reveal.mwindow.boxObject.width;
			}

			Reveal.selected = current;
			Reveal.positionPreviews();
			break;
		}
	},

	handleKeys: function (event)
	{
		var stopEvent = true;

		switch (event.keyCode) {
		case event.DOM_VK_DOWN:
			Reveal.selectPreview(true);
			break;

		case event.DOM_VK_UP:
			Reveal.selectPreview(false);
			break;

		case event.DOM_VK_PAGE_DOWN:
			Reveal.pagePreviews(true);
			break;

		case event.DOM_VK_PAGE_UP:
			Reveal.pagePreviews(false);
			break;

		case event.DOM_VK_INSERT:
			if (event.ctrlKey)
				Reveal.toggleAll();
			else
				Reveal.toggleHistory();
			break;

		case event.DOM_VK_ENTER:
		case event.DOM_VK_RETURN:
			if (Reveal.selected != null)
				Reveal.previewClick(Reveal.selected, event);
			break;

		case event.DOM_VK_ESCAPE:
			Reveal.closeReveal();
			break;

		// err, bad idea..
		case event.DOM_VK_DELETE:
			if (event.shiftKey && Reveal.selected != null) {
				var browser = Reveal.selected.owner_hentry.src.browser;
				var i, tabbox;
				tabbox = getBrowser().mTabs;
				for (i in tabbox) {
					if (tabbox[i].linkedBrowser == browser) {
						getBrowser().removeTab(tabbox[i]);
						break;
					}
				}
			}
			break;

		default:
			stopEvent = false;
			break;
		}

		if (stopEvent) {
			event.stopPropagation();
			return false;
		}

		return true;
	},

	addtabUpdate: function (browser)
	{
		this.generateTNElement(browser);
		this.positionPreviews();
	},

	removetabUpdate: function (browser)
	{
		var index;
		for (index in this.tabthumbs)
			if (this.tabthumbs[index].owner_hentry.src.browser == browser)
				break;

		if (this.tabthumbs[index].owner_hentry.src.browser != browser)
			return;

		var tmp = this.tabthumbs[index];
		var old_index = this.current_thumbs.indexOf(tmp);
		this.tabthumbs.splice(index, 1);
		// TODO: FIXME, there should always be a selection
		if (this.selected != null &&
		    this.lastmode != "all" &&
		    tmp.owner_hentry.src == this.selected.owner_hentry.src) {
			if (this.tabthumbs != this.current_thumbs) {
				this.lastmode = "tab";
				this.modedrop.value = "tab";
				this.syncCurrentThumbs(this.tabthumbs);
			} else
				this.bbar.removeChild(tmp);
		} else if (this.tabthumbs == this.current_thumbs)
			this.bbar.removeChild(tmp);
		else if (this.lastmode == "all")
			this.syncCurrentThumbs(this.getAllArray());

		if (tmp == this.selected)
			this.selected = null;

		if (tmp == this.current_tab) {
			this.current_tab = null;
			this.closeReveal();
		}

		this.distColors();
		if (this.tabthumbs == this.current_thumbs)
			this.positionPreviews();

		if (this.current_thumbs[old_index] != null) {
			this.selected = this.current_thumbs[old_index];
			this.showSelected();
		} else if (this.current_thumbs[old_index - 1] != null) {
			this.selected = this.current_thumbs[old_index - 1];
			this.showSelected();
		} else
			this.selectPreview(true);
	},

	reordertabUpdate: function (browser)
	{
		if (this.tabthumbs == this.current_thumbs) {
			this.tabthumbs = this.getTabArray();
			this.syncCurrentThumbs(this.tabthumbs);
		} else if (this.lastmode == "all") {
			this.syncCurrentThumbs(this.getAllArray());
			this.tabthumbs = this.getTabArray();
		} else
			this.tabthumbs = this.getTabArray();
	},

	thumbnailUpdate: function (browser, fh)
	{
		var index = getBrowser().browsers.indexOf(browser);
		this.generateTNElement(browser);
		if (fh && this.current_thumbs.indexOf(this.tabthumbs[index]) < 0) {
			this.current_thumbs = this.getHistoryArray();
			this.syncCurrentThumbs(this.current_thumbs);
		}

		this.positionPreviews();
	},

	observe: function (aSubject, aTopic, aData)
	{
		if (aTopic != "nsPref:changed")
			return;

		switch (aData) {
		case "reveal.hide_tabbar":
			// setting hidden seems to screw things up,
			// so we set style.visibility. *shrug*
			if (this.prefs.getBoolPref("hide_tabbar"))
				getBrowser().mStrip.style.visibility = "collapse";
			else
				getBrowser().mStrip.style.removeProperty("visibility");
			break;
		case "reveal.spiffy":
			this.spiff = this.prefs.getBoolPref("spiffy");
			break;
		case "reveal.stack_preview":
			this.stack = this.prefs.getBoolPref("stack_preview");
			this.positionPreviews();
			break;
		}
	},

	onLoad: function ()
	{
		this.current_thumbs = this.tabthumbs;
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.");
		this.stack = this.prefs.getBoolPref("stack_preview");
		this.spiff = this.prefs.getBoolPref("spiffy");
		if (this.prefs.getBoolPref("hide_tabbar"))
			getBrowser().mStrip.style.visibility = "collapse";
		else
			getBrowser().mStrip.style.removeProperty("visibility");
		Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).addObserver("reveal.", Reveal, false);

		this.modedrop = document.getElementById("reveal-modes");
		this.finddrop = document.getElementById("reveal-findmodes");

		this.findmode = document.getElementById("reveal-findmodes").value;
		this.regex = document.getElementById("reveal-find-regex").checked;
		this.match_case = document.getElementById("reveal-find-case").checked;
		document.getElementById("reveal-find-textbox").inputField.addEventListener("keypress", this.handleKeys, true);
		this.reveal_bc = document.getElementById("viewReveal");
		this.reveal_bc.hidden = true;
		this.mwindow = document.getElementById("main-window");
		this.findbar = document.getElementById("reveal-findbar");
		this.bbar = document.getElementById("reveal-backbar");
		this.bbar.addEventListener("DOMMouseScroll",
			function(e) {
			var delta;
			if (e.detail >= 0) {
				if (Reveal.hscroll <= -Reveal.maxhscroll)
					return;

				delta = Reveal.maxhscroll + Reveal.hscroll;
				if (delta > 30*e.detail)
					delta = 30*e.detail;
			} else {
				if (Reveal.hscroll >= 0)
					return;

				delta = Reveal.hscroll;
				if (delta < 30*e.detail)
					delta = 30*e.detail;
			}

			Reveal.scrollPreviews(-delta);
			}, false);

		this.bbar.addEventListener("mousedown", function (event) {
			if (event.target != Reveal.bbar)
				return;

			this.style.cursor = "-moz-grabbing";
			Reveal.grab_x = event.screenX;
			Reveal.dragging = true;
			Reveal.mwindow.addEventListener("mousemove", Reveal.dragBackbar, true);
			Reveal.mwindow.addEventListener("mouseup", Reveal.deactivateDrag, true);
		}, false);

		HistoryThumb.addTabListener(this);
		HistoryThumb.addThumbListener(this);
		this.generateTNElement(getBrowser().selectedBrowser);
		window.addEventListener("resize", function (e) {Reveal.positionPreviews();}, false);
	},

	onUnload: function ()
	{
		HistoryThumb.removeThumbListener(this);
		HistoryThumb.removeTabListener(this);
		Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).removeObserver("reveal.", this);
	}
};

window.addEventListener("load", function(e) { Reveal.onLoad(); }, false);
window.addEventListener("unload", function(e) { Reveal.onUnload(); }, false);

